﻿/**************************************************************************************************
 * Helper:   Generic exception handler for exceptions thrown when accessing the Crm web service.
 * Purpose:  Identifies the most common exceptions and creates a readable output that can be
 *           directly logged to a log file, a consolw window or message box. It's not meant to be
 *           be included in your own applications but should be well suited for testing.
 * Date:     16 Feb 2006
 * Author:   Michael Höhne, mailto:michael.hoehne@stunnware.com
 * DevEnv:   Microsoft Visual Studio 2003
 * 
 * This class is used across all samples. I needed to replace the string.IsNullOrEmpty method from
 * the .NET 2.0 Framework with the standard (str == null) || (str.Length == 0), but everything else is
 * the same as in the VS2005 samples.
 */

using System;
using System.Net;
using System.Text;
using System.Web.Services.Protocols;

namespace Stunnware.Crm.Samples {

   class CrmExceptionHandler {

      //The source exception passed to the constructor
      private Exception _exception;
      
      //Detailed error information. This member is set in the constructor if details exists.
      private string _details;
      
      //Recommended actions to solve the error, if the reason can clearly be identifed. Set in
      //the constructor.
      private string _recommendedAction;

      public CrmExceptionHandler(Exception e) {

         _exception = e;

         if (e is SoapException) {

            SoapException soapException = e as SoapException;

            //Usually, a SoapException is thrown when an error occurs. The SoapException has a 
            //Detail property containing detailed error information. You should always catch it and 
            //always evaluate the Detail property. It will save you a lot of time. 
            _details = soapException.Detail.InnerXml;
         }

         else if (e is WebException) {

            WebException webException = e as WebException;

            //Another common exception is the WebException. It indicates that your request did not
            //reach the end point or the whoAmIResponse could not be sent to you. It is very useful to check
            //the Status property, as this can be used programatically to perform specific actions.
            //Say that e.Status equals System.Net.WebExceptionStatus.NameResolutionFailure. You then 
            //know that you either have a DNS problem or the server specified in the service Url is
            //incorrect.
            _details = "Status = " + webException.Status;

            if (webException.Status == WebExceptionStatus.NameResolutionFailure) {

               _recommendedAction = 
                  "The specified machine name could not be resolved. It maybe misspelled in the " +
                  "web service url or you need to setup an additional DNS server in your IP " +
                  "configuration.";
            }

            else if (webException.Status == WebExceptionStatus.ConnectFailure) {

               _recommendedAction = 
                  "The server specified in the web service url either does not exist, does not " +
                  "host the web service or the IIS service is currently not running.";
            }


            else if (webException.Status == WebExceptionStatus.ProtocolError) {

               _recommendedAction =
                  "Please check your supplied credentials. Though the protocol error may have " +
                  "other reasons, incorrect credentials are a common error reason.\r\n" +
                  "If you receive this error using the CredentialCache.DefaultCredentials for the " +
                  "Credentials property of the web service, try the following instead:\r\n" +
                  "new System.Net.NetworkCredential(\"username\", \"password\", \"domain\");\r\n" +
                  "For testing purposes you can specify the credentials in code, but in a real " +
                  "application you must provide some logon dialog to let the user enter the " +
                  "correct logon information.";
            }
         }

         else {
            //We do not have any details or a recommendation for exceptions that were not handled 
            //above.
            _details = string.Empty;
            _recommendedAction = string.Empty;
         }
      }

      /// <summary>
      /// Gets the source exception.
      /// </summary>
      public Exception Exception {
         get {
            return _exception; 
         }
      }

      /// <summary>
      /// Gets details about the exception, if available.
      /// </summary>
      public string Details {
         get {
            return _details;
         }
      }

      /// <summary>
      /// Gets some assistance on how to solve the issue, if the error reason could be 
      /// identified.
      /// </summary>
      public string RecommendedAction {
         get {
            return _recommendedAction;
         }
      }

      /// <summary>
      /// Returns a string representation of the collected information. You can use it directly to
      /// write the exception, exception details and recommended action to a log file. 
      /// </summary>
      /// <returns>A string with all information we found.</returns>
      public override string ToString() {

         StringBuilder s = new StringBuilder();

         //Start with a general header, including the exception type name. You may remove this line
         //as it's also listed in _exception.ToString() at the end of this method.
         s.AppendFormat(
            "A {0} occurred while executing the requested operation.\r\n", 
            _exception.GetType().Name);

         //Append the exception message.
         s.AppendFormat("   Message: {0}\r\n", _exception.Message);

         //If we have found detailed information about the error reason, we append it to the
         //output.
         if ((_details != null) && (_details.Length > 0)) {
            s.AppendFormat("   Details: {0}\r\n", _details);
         }

         //Further, if we can recommend any specific action to solve the issue, we do it here.
         if ((_recommendedAction != null) && (_recommendedAction.Length > 0)) {
            s.AppendFormat("   Action:  {0}\r\n", _recommendedAction);
         }

         //Finally, append the complete exception, including inner exceptions and the stack trace
         s.Append("\r\nException\r\n");
         s.Append(_exception.ToString());

         return s.ToString();
      }
   }
}
