﻿/**************************************************************************************************
 * Helper:   CrmService wrapper. 
 * Purpose:  Contains all methods that have been discussed in previous examples, allowing upcoming
 *           sample code to be much smaller in size and more clear about the new stuff. 
 * Date:     28 Jan 2006
 * Author:   Michael Höhne, mailto:michael.hoehne@stunnware.com
 * DevEnv:   Microsoft Visual Studio 2005
 * 
 * This class replaces the CrmService initialization code and Execute method found in the WhoAmI
 * sample. It also contains a WhoAmI method executing a WhoAmIRequest.
 */

using System;
using System.Net;
using Stunnware.Crm.Samples.CrmService3;

namespace Stunnware.Crm.Samples {

   class CrmServiceHelper {

      CrmService _crmService;

      /// <summary>
      /// Creates and initializes a new CrmService object.
      /// </summary>
      /// <param name="webServiceUrl">The Crm web service url. Cannot be null.</param>
      /// <param name="credentials">The credentials to use when connecting to the web
      /// service. Cannot be null.</param>
      public CrmServiceHelper(string webServiceUrl, ICredentials credentials) {

         if ((webServiceUrl == null) || (webServiceUrl.Length == 0)) {
            //You must specify a web service url. As the default url in the web reference points
            //to my test system, you cannot use it. If you have changed the web reference to use
            //your own system instead, please pass the full url to this constructor.
            throw new ArgumentNullException("You must provide the web service url.");
         }

         if (credentials == null) {
            //You must pass valid credentials to the constructor. You can use
            //System.Net.CredentialCache.DefaultCredentials or a NetworkCredential.
            throw new ArgumentNullException("You must specify the logon credentials.");
         }

         //The first step to access the Crm Server is the creation and initialization of a
         //CrmService object. You will use it for all operations, including read, create and
         //update operations.
         _crmService = new CrmService();

         //Set the web service url.
         _crmService.Url = webServiceUrl;

         //Set the credentials
         _crmService.Credentials = credentials;

         //PreAuthenticate should be set to true, to avoid authentication overhead when calling
         //web service methods. See WebClientProtocol.PreAuthenticate in the MSDN for more 
         //information.
         _crmService.PreAuthenticate = true;
      }

      /// <summary>
      /// Returns the initialized CrmService object.
      /// </summary>
      public CrmService Service {
         get {
            return _crmService;
         }
      }

      /// <summary>
      /// The CrmService.Execute method uses the type of the request to find out what your request
      /// is. Though some requests (like the WhoAmIRequest) do not need any parameters, they must
      /// be specified as a separate class to let the server distinguish between the requests.
      /// The Execute method understands any request. You can find all of them in the reference.cs
      /// file: Navigate to the Request class and note the gigantic list of XmlInclude attributes.
      /// All of these includes specify a unique request. If you have managed to know what each of
      /// them does, you don't need to read any further.
      /// </summary>
      /// <param name="request">The request to be executed by the Crm Server.</param>
      /// <returns>The whoAmIResponse sent back from the Crm Server. You have to explicitly cast the 
      /// return value to the appropriate whoAmIResponse.
      /// It's an easy task, as each whoAmIResponse is named like the corresponding request.</returns>
      public Response Execute(Request request) {
         return _crmService.Execute(request);
      }

      public WhoAmIResponse WhoAmI() {

         //The WhoAmIRequest has no properties, so you can also call 
         //crmService.Execute(new WhoAmIRequest()), but to make the code more readable, I avoid 
         //these constructs.
         WhoAmIRequest request = new WhoAmIRequest();

         //Execute the WhoAmIRequest.
         return (WhoAmIResponse) Execute(request);
      }
   }
}
