/**************************************************************************************************
 * Sample:   FetchXML Library Test.
 * 
 * Purpose:  This test application defines some basic and some enhanced queries using the FetchXML 
 *           library. It also contains two helper classes (CrmEntity and CrmEntityCollection) that
 *           you can use to easily parse the result of a fetch query. 
 * 
 * Date:     07 Jan 2007
 * Author:   Michael Hhne, mailto:michael.hoehne@stunnware.com
 * DevEnv:   Microsoft Visual Studio 2003
 *  
 * Please open View\Task List and select "Comments" in the task list window to see a list of actions
 * you have to perform in order to get this sample running in your test environment.
 */
using System;
using System.Net;
using stunnware.CRM.Fetch;
using Stunnware.Crm.Samples.CrmService3;

namespace Stunnware.Crm.Samples {

	class FetchTest {

		//TODO: The CityToSearch constant defines the city used to filter data in a variety of samples.
		//The filter criteria is account.address1_city =  CityToSearch. If you have some accounts located
		//in Seattle, change it to "Seattle". 
		const string CityToSearch = "Grasbrunn";

		static string RowDelimiter = new string('-', 79);
		

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args) {

			//The CrmServiceHelper does the initialization of the CrmService object. 
			//TODO: Change the server name "stunnwarecrm" to the name or ip address of your Crm
			//Server machine.
			//TODO: If you want to update the web reference, you must also change the server name 
			//in the CrmService3 web reference.
			//TODO: If you receive an authentication error when trying to connect to the web service,
			//you should try to use a NetworkCredential instead of the DefaultCredentials.
			CrmServiceHelper serviceHelper = new CrmServiceHelper(
				"http://stunnwarecrm/mscrmservices/2006/crmservice.asmx",
				CredentialCache.DefaultCredentials);

         //Any call to the Execute method can throw an exception, so it's always a good idea to
         //catch them if you're able to perform a correct error handling.
			try {

				//Simple test to see if we are authenticated. If we pass that line without an 
				//exception being thrown, out credentials were accepted by the Crm server.
				WhoAmIResponse whoAmIResponse = (WhoAmIResponse) serviceHelper.WhoAmI();

				//Get the real CrmService instance from the helper class.
				CrmService crmService = serviceHelper.Service;

				//The following methods define some FetchXML queries. Look at the method comments for
				//further descriptions.
				AllAccounts(crmService);
				First10Accounts(crmService);
				First10AccountsWithColumnSet(crmService);
				First10AccountsInACity(crmService);
				ContactsOfAccountsInACity(crmService);
				AccountsAndContactsOfAccountsInACity(crmService);
				AccountsAndContactsOfAccountsInACityWithAlias(crmService);
				OrderedAccountsAndContactsOfAccountsInACityWithAlias(crmService);
				CountAccounts(crmService);
			}

			catch (Exception e) {
				//Again, a helper class. The exception handling in the samples will be mostly the same,
				//so I decided to create another helper class that takes an exception and produces
				//a readable output that can directly be used for logging purposes.
				CrmExceptionHandler exceptionHandler = new CrmExceptionHandler(e);
				Console.WriteLine(exceptionHandler.ToString());
			}

			//The following lines are helpful when running the code inside of Visual Studio. It allows
			//you to watch the output in the console window before it closes.
			Console.WriteLine("Done. Press Enter to exit.");
			Console.ReadLine();
		}

		/// <summary>
		/// This sample selects all accounts in the system. If you have a lot of accounts, do not
		/// run it, as it also returns every single attribute. I just include it here as it's the
		/// easiest query you can build.
		/// </summary>
		/// <param name="service">An initialized CrmService object.</param>
		static void AllAccounts(CrmService service) {

			FetchStatement query = new FetchStatement();
			query.Entity = new FetchEntity("account");
			query.Entity.Columns = new FetchColumnSet(true);

			//If you want to see the FetchXML query, uncomment the next line.
			//string fetchXml = query.ToString();

			string responseXml = service.Fetch(query.ToString());

			CrmEntityCollection result = new CrmEntityCollection(responseXml);
			DisplayResults(result, "All Accounts");
		}

		/// <summary>
		/// This sample selects the first 10 accounts in the system. It's the same as <see cref="AllAccounts"/>
		/// but limits the result to 10 records.
		/// </summary>
		/// <param name="service">An initialized CrmService object.</param>
		static void First10Accounts(CrmService service) {

			FetchStatement query = new FetchStatement();
			query.Entity = new FetchEntity("account");
			query.Entity.Columns = new FetchColumnSet(true);
			query.PageCount = 10;

			//If you want to see the FetchXML query, uncomment the next line.
			//string fetchXml = query.ToString();

			string responseXml = service.Fetch(query.ToString());

			CrmEntityCollection result = new CrmEntityCollection(responseXml);
			DisplayResults(result, "First 10 Accounts");
		}

		/// <summary>
		/// This sample selects some chosen attributes of the first 10 accounts in the system. 
		/// It's the same as <see cref="First10Accounts"/> but uses a column set to only return 
		/// a limited number of columns.
		/// </summary>
		/// <param name="service">An initialized CrmService object.</param>
		static void First10AccountsWithColumnSet(CrmService service) {

			FetchStatement query = new FetchStatement();
			query.Entity = new FetchEntity("account");
			query.Entity.Columns = new FetchColumnSet(new string[] { "name", "address1_city", "address1_country" });
			query.PageCount = 10;

			//If you want to see the FetchXML query, uncomment the next line.
			//string fetchXml = query.ToString();
			string responseXml = service.Fetch(query.ToString());

			CrmEntityCollection result = new CrmEntityCollection(responseXml);
			DisplayResults(result, "First 10 Accounts with column set");
		}

		/// <summary>
		/// This sample is based on <see cref="First10AccountsWithColumnSet"/>. It adds a simple
		/// condtion to only return the accounts located in in a specific city.
		/// </summary>
		/// <param name="service">An initialized CrmService object.</param>
		static void First10AccountsInACity(CrmService service) {

			FetchCondition condition = new FetchCondition("address1_city");
			condition.ConditionOperator = FetchConditionOperator.Equal;
			condition.Values.Add(CityToSearch);

			FetchStatement query = new FetchStatement();
			query.Entity = new FetchEntity("account");
			query.Entity.Columns = new FetchColumnSet(new string[] { "name", "address1_city", "address1_country" });
			query.PageCount = 10;
			query.Entity.Filter = new FetchFilter(FetchFilterType.And);
			query.Entity.Filter.Conditions.Add(condition);

			//If you want to see the FetchXML query, uncomment the next line.
			//string fetchXml = query.ToString();
			string responseXml = service.Fetch(query.ToString());

			CrmEntityCollection result = new CrmEntityCollection(responseXml);
			DisplayResults(result, "First 10 Accounts in " + CityToSearch);
		}

		/// <summary>
		/// This sample returns all contacts associated to an account located in a specific city.
		/// </summary>
		/// <param name="service">An initialized CrmService object.</param>
		static void ContactsOfAccountsInACity(CrmService service) {

			FetchCondition condition = new FetchCondition("address1_city");
			condition.ConditionOperator = FetchConditionOperator.Equal;
			condition.Values.Add(CityToSearch);

			FetchLinkEntity linkContactToAccounts = new FetchLinkEntity("account");
			linkContactToAccounts.From = "accountid";
			linkContactToAccounts.To = "parentcustomerid";
			
			linkContactToAccounts.Filter = new FetchFilter(FetchFilterType.And);
			linkContactToAccounts.Filter.Conditions.Add(condition);

			FetchStatement query = new FetchStatement();
			query.Entity = new FetchEntity("contact");
			query.Entity.Columns = new FetchColumnSet(new string[] { "fullname" });
			query.Entity.LinkEntities.Add(linkContactToAccounts);

			//If you want to see the FetchXML query, uncomment the next line.
			//string fetchXml = query.ToString();
			string responseXml = service.Fetch(query.ToString());

			CrmEntityCollection result = new CrmEntityCollection(responseXml);
			DisplayResults(result, "All Contacts of Accounts in " + CityToSearch);
		}

		/// <summary>
		/// This sample is an extension to the <see cref="ContactsOfAccountsInACity"/> sample.
		/// It returns all contacts associated to an account located in a specific city as before, 
		/// but the result also includes some attributes of the account entity.
		/// </summary>
		/// <param name="service">An initialized CrmService object.</param>
		static void AccountsAndContactsOfAccountsInACity(CrmService service) {

			FetchCondition condition = new FetchCondition("address1_city");
			condition.ConditionOperator = FetchConditionOperator.Equal;
			condition.Values.Add(CityToSearch);

			FetchLinkEntity linkContactToAccounts = new FetchLinkEntity("account");
			linkContactToAccounts.From = "accountid";
			linkContactToAccounts.To = "parentcustomerid";
			linkContactToAccounts.Columns = new FetchColumnSet(new string[] { "name", "address1_city", "address1_country" });
			
			linkContactToAccounts.Filter = new FetchFilter(FetchFilterType.And);
			linkContactToAccounts.Filter.Conditions.Add(condition);

			FetchStatement query = new FetchStatement();
			query.Entity = new FetchEntity("contact");
			query.Entity.Columns = new FetchColumnSet(new string[] { "fullname" });
			query.Entity.LinkEntities.Add(linkContactToAccounts);

			//If you want to see the FetchXML query, uncomment the next line.
			//string fetchXml = query.ToString();
			string responseXml = service.Fetch(query.ToString());

			CrmEntityCollection result = new CrmEntityCollection(responseXml);
			DisplayResults(result, "All Contacts of Accounts in " + CityToSearch);
		}

		/// <summary>
		/// This sample is the same as <see cref="AccountsAndContactsOfAccountsInACity"/> but it uses
		/// an alias to return the account properties as account.name instead of parentcustomerid.name.
		/// </summary>
		/// <param name="service">An initialized CrmService object.</param>
		static void AccountsAndContactsOfAccountsInACityWithAlias(CrmService service) {

			FetchCondition condition = new FetchCondition("address1_city");
			condition.ConditionOperator = FetchConditionOperator.Equal;
			condition.Values.Add(CityToSearch);

			FetchLinkEntity linkContactToAccounts = new FetchLinkEntity("account");
			linkContactToAccounts.From = "accountid";
			linkContactToAccounts.To = "parentcustomerid";
			linkContactToAccounts.Columns = new FetchColumnSet(new string[] { "name", "address1_city", "address1_country" });
			linkContactToAccounts.Alias = "account";
			
			linkContactToAccounts.Filter = new FetchFilter(FetchFilterType.And);
			linkContactToAccounts.Filter.Conditions.Add(condition);

			FetchStatement query = new FetchStatement();
			query.Entity = new FetchEntity("contact");
			query.Entity.Columns = new FetchColumnSet(new string[] { "fullname" });
			query.Entity.LinkEntities.Add(linkContactToAccounts);

			//If you want to see the FetchXML query, uncomment the next line.
			//string fetchXml = query.ToString();
			string responseXml = service.Fetch(query.ToString());

			CrmEntityCollection result = new CrmEntityCollection(responseXml);
			DisplayResults(result, "All Contacts of Accounts (with Alias) in " + CityToSearch);
		}

		/// <summary>
		/// This sample is the same as <see cref="AccountsAndContactsOfAccountsInACityWithAlias"/> but 
		/// orders the result by lastname and firstname.
		/// </summary>
		/// <param name="service">An initialized CrmService object.</param>
		static void OrderedAccountsAndContactsOfAccountsInACityWithAlias(CrmService service) {

			FetchCondition condition = new FetchCondition("address1_city");
			condition.ConditionOperator = FetchConditionOperator.Equal;
			condition.Values.Add(CityToSearch);

			FetchLinkEntity linkContactToAccounts = new FetchLinkEntity("account");
			linkContactToAccounts.From = "accountid";
			linkContactToAccounts.To = "parentcustomerid";
			linkContactToAccounts.Columns = new FetchColumnSet(new string[] { "name", "address1_city", "address1_country" });
			linkContactToAccounts.Alias = "account";
			
			linkContactToAccounts.Filter = new FetchFilter(FetchFilterType.And);
			linkContactToAccounts.Filter.Conditions.Add(condition);

			FetchStatement query = new FetchStatement();
			query.Entity = new FetchEntity("contact");
			query.Entity.Columns = new FetchColumnSet(new string[] { "fullname" });
			query.Entity.LinkEntities.Add(linkContactToAccounts);

			FetchOrder orderByLastName = new FetchOrder("lastname");
			FetchOrder orderByFirstName = new FetchOrder("firstname");

			query.Entity.Orders.Add(orderByLastName);
			query.Entity.Orders.Add(orderByFirstName);

			//If you want to see the FetchXML query, uncomment the next line.
			//string fetchXml = query.ToString();
			string responseXml = service.Fetch(query.ToString());

			CrmEntityCollection result = new CrmEntityCollection(responseXml);
			DisplayResults(result, "All Contacts of Accounts (with Alias, ordered) in " + CityToSearch);
		}

		/// <summary>
		/// This sample returns the total number of accounts in the system. 
		/// </summary>
		/// <param name="service">An initialized CrmService object.</param>
		static void CountAccounts(CrmService service) {

			FetchStatement query = new FetchStatement();

			//Must be set to true when using an aggregate.
			query.Aggregate = true;
			query.Entity = new FetchEntity("account");

			//When specifying an aggregate, you need to pass a valid attribute name in the column set.
			query.Entity.Columns = new FetchColumnSet(new string[] { "accountid" });
			
			//The only supported aggregate is Count and we're using it in this sample.
			query.Entity.Columns.Aggregate = FetchAggregate.Count;

			//You also *must* specify an alias when using an aggregate. 
			query.Entity.Columns.Alias = "Count";

			//If you want to see the FetchXML query, uncomment the next line.
			//string fetchXml = query.ToString();

			string responseXml = service.Fetch(query.ToString());

			CrmEntityCollection result = new CrmEntityCollection(responseXml);
			DisplayResults(result, "Total number of Accounts");
		}

		/// <summary>
		/// Displays all attributes of all entities contained in an entity collection.
		/// </summary>
		/// <param name="entities">A collection of entities.</param>
		/// <param name="title">A description of the result set.</param>
		static void DisplayResults(CrmEntityCollection entities, string title) {

			Console.WriteLine();
			Console.WriteLine(RowDelimiter);
			Console.WriteLine("Results of " + title);
			Console.WriteLine(RowDelimiter);

			int index = 1;

			foreach(CrmEntity entity in entities) {
				Console.WriteLine("Entity #{0}", index++);
				Console.WriteLine(RowDelimiter);

				if (entity.AttributeCount > 0) {
					foreach(string attributeName in entity.AttributeNames) {
						Console.WriteLine("{0} = {1}", attributeName, entity.GetAttributeDisplayValue(attributeName));
					}
				}
			}
		}
	}
}